import java.util.concurrent.*;
import java.util.Vector;


/**
 * A class that holds the AlgorithmHandlers for all algorithms 
 * and handles synchronization of the algorithms when they are run.
 *
 * This class does not have to be changed
 * 
 * @author Sara Stymne
 *
 */
class Algorithms {
	
	/**
	 * The vector of integers to be sorted
	 */
	private Vector<AlgorithmHandler> algorithms;
	
	/**
	 * The barrier that handles synchronization of algorithms
	 */
	CyclicBarrier barrier;
	
	/**
	 * The status of the algorithms
	 */
	private AlgorithmStatus status;
	
	/**
	 * The time that the animation pauses between each step.
	 * It is initialised to 50 (milliseconds)
	 */
	public int delayTime = 50;
	
	/**
	 * Creates a new instance of Algorithms with no algorithms
	 */
	public Algorithms() {
		algorithms = new Vector<AlgorithmHandler>();
		status = new AlgorithmStatus();
	}
	
		
	/**
	 * Creates a new AlgorithmHandler which it then adds to itself.
	 * 
	 * @param alg the actual new algorithm
	 * @param vec the sortVector of the new algorithm
	 */
	public void add(SortAlgorithm alg, SortVector vec) {
		algorithms.add(new AlgorithmHandler(alg, vec, barrier, status));
	}
	
	/**
	 * Updates all AlgorithmHandlers with a copy of a new sort vector.
	 * 
	 * @param sortVec the new sort vector
	 */
	public void setVector(SortVector sortVec) {
		
		for(AlgorithmHandler alg: algorithms) {
			SortVector vectorClone = sortVec.clone();
			alg.setVector(vectorClone);
		}	
	}	
	
	/**
	 * Returns the widget of the last AlgorithmHandler that was added.
	 * 
	 * @return the widget
	 */
	public AnimationWidget getLastWidget() {
		return algorithms.lastElement().getWidget();
	}
	
	/**
	 * Starts the animation for all AlgorithmHandlers, 
	 * which includes setting up the barrier.
	 * 
	 * @param swapWeight the swapWeight for this run of the animation
	 */
	void start(int swapWeight) {
		
		//Gives status the current values
		status.setNumberOfAlgorithms(algorithms.size());
		status.resetFinishedAlgorithms();
		
		//Creates a new CyclicBarrier
        //Which assures that all algorithms are synchronized
        //Waits delayTime ms between each step of all algorithms
		barrier = new CyclicBarrier(status.getNumberOfAlgorithms(), 
				                    new Runnable() {
		                                public void run() {				
		                                    try {  
		                                        Thread.sleep(delayTime);
		                                    }
		                                    catch(InterruptedException e) {}
		                                }
		                            });
		
		//Adds a reference of the barrier 
		//and the current swapWeight to each algorithm
		for (AlgorithmHandler alg: algorithms) {
			alg.setBarrier(barrier);
			alg.setSwapWeight(swapWeight);
		}
		
		//Starts all the algorithms
		for (AlgorithmHandler alg: algorithms) {
			new Thread(alg).start();
		}
		
	}
	
	/**
	 * Checks if the animation is finished, i.e. if no animation currently runs
	 * 
	 * @return true if the animation is finished, false if it runs
	 */
	public boolean animationFinished() {
		return status.animationFinished();
	} 
	
	
	/**
	 * Sets the delay time 
	 * that decides how long the break between each step of the animation is
	 * 
	 * @param dTime the new delay time
	 */
	public void setDelayTime(int dTime) {
        delayTime = dTime;
	}
}

